/**

 * Copyright (C) 2001-2017, Foxit Software Inc..

 * All Rights Reserved.

 *

 * http://www.foxitsoftware.com

 *

 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..

 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 

 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.

 *

 * This file is an example to demonstrate how to process a number of PDF documents in multiple threads environment.

 */



//Include common header file.

#include "../comm_src/fgsdk_common.h"

#include<sys/types.h>
 
#include<sys/wait.h>
#include<time.h>
#include<sys/time.h>


//process procedure to process PDF document

void Process_Proc(string inputFileName)

{
	//Define a string object to store log data
	string strLog("\r\n====== ");
	//Get PDF file name from parameter, and output to log
	string strFile = FSDK_GetFixFolder();
	strFile += "/";

	strFile += inputFileName;

	strLog  += inputFileName;

	//Compose the output file
	string strTxtFile = FSDK_GetOutputFilesFolder("linux_mt_pdf2txt") + "/";
	strTxtFile += string(inputFileName.c_str(), inputFileName.length()-4) + ".txt";
	printf("text floder = %s\n",strTxtFile.c_str());
	//Open an output txt file
	CAutoDocument autoOutPutDoc(strTxtFile.c_str(), "wb+");
	
	//Get current time
	unsigned int nStartTime = FSDK_GetClock();
	CAutoDocument autoPdfDoc(strFile.c_str(), "rb");
	//Load PDF file
	FS_RESULT ret = autoPdfDoc.LoadPDFDocument(NULL);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return;
	}

	//Load successfully

	strLog += " success\r\n";
	//Get count of PDF pages

	FS_INT32 pageCount = 0;

	ret = FSPDF_Doc_CountPages(autoPdfDoc.GetDocument(), &pageCount);

	if (ret == FSCRT_ERRCODE_SUCCESS)
	{

		FS_INT32 pageIndex = 0;

		do
		{

		//Get a PDF page by index

		CAutoPDFPage autoPage;
		ret =autoPage.LoadPDFPage(autoPdfDoc.GetDocument(),pageIndex);

		//Before loading a text page from a PDF page, the PDF page should be pared successfully first.
		if (FSCRT_ERRCODE_SUCCESS != autoPage.ParsePage())

		{
			pageIndex ++;
			continue;
	
		}

		//Load a text page based on a PDF page

		ret = autoPage.LoadTextPage();

		if (FSCRT_ERRCODE_SUCCESS != ret)

		{
			pageIndex ++;
			continue;
		}

		//Export a text page to the given text file

		ret = FSPDF_TextPage_ExportToFile(autoPage.GetPDFTextPage(),autoOutPutDoc.GetFileHander());

		//	printf("test ret = %d\n",ret);

		if(ret != FSCRT_ERRCODE_SUCCESS)
		{
			FSDK_OutputLog("Failure: In function FSPDF_TextPage_ExportToFile with error code %d.\r\n", ret);

			pageIndex ++;
			continue;

		}
					
					
		//Move to next page
		pageIndex ++;

		} while (pageIndex < pageCount);

	}  
}



//Entrance of this example
#define  MuilProcess
#define  processNum  4
int main(int argc, char* argv[])

{
 

    struct timeval tv_start;
    struct timeval tv_end;

    gettimeofday(&tv_start, NULL);

	//Set executing file path

	FSDK_SetExecuteFilePath(argv[0]);

	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("linux_mt_pdf2txt");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: linux_mt_pdf2txt\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
	//Failed to initialize, output status and return
		FSDK_CloseLog();
		return -1;
	}
	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	string inputFilesPath = FSDK_GetFixFolder();
	FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
	FSDK_OutputLog("Output to folder: %s\r\n", logPath.c_str());
	//Get folder of inputting files for this example
	FSDK_OutputLog("\r\nEnumerate files in folder input_files/linux_mt_pdf2txt: ");
	//string inputFilesPath = FSDK_GetInputFilesFolder("linux_mt_pdf2txt");
	//Open folder to enumerate files
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());
	int id[processNum] = {0};
	if (folder)
	{
		//Array to store thread handles
		vector<void*> threads;
		int fileIndex = 0;
		int index = 0;
		
		do
		{
			//Get a file object from folder
			FS_BOOL isDir = FALSE;
			string strFile = FSDK_GetNextFile(folder, &isDir);
			if (strFile.size() > 0 && !isDir)
			{
				//Check whether is a PDF file
				if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
				{

					fileIndex ++;
				#ifdef MuilProcess
					int pid = fork();
				#else
					int pid = 0;
				#endif
					
				        if(pid == 0)
				        {
				            Process_Proc(strFile);
					 #ifdef MuilProcess
					    printf("exit***************\n");
				            exit(0);
					 #endif
				        }
				        else
				        {
				             printf("parent process!\n");
				        }
			#ifdef MuilProcess
					if(fileIndex < processNum) 
					{
						id[fileIndex] = pid;
						continue;
				      	}
					else
						id[index] = pid;
				       
					while(1)
					{
						int flag = false;
						
						for( int j = 0; j < processNum; j++)
						{
						   pid = waitpid(id[j], NULL, WNOHANG);
						   if(pid !=0)
						   {
						      index = j;
						      flag = true;
						      printf("pid = %d end\n" ,pid);
						      break;
						   }
						}
						if(flag) break;
					}
				#endif
				     

				}
				//Go on for next file
				continue;
			}
			//If it's an invalid file name
			if (strFile.size() < 1) break;
		} while (1);
		//Close folder

		FSDK_CloseFolder(folder);
	}

      for(int i = 0; i < processNum; i++)
      {
	 printf("wait....finish\n");
	 #ifdef MuilProcess
         int pid = waitpid(id[i], NULL, 0);
	 #endif
      }

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();

    gettimeofday(&tv_end, NULL);

    if(tv_end.tv_usec >= tv_start.tv_usec)
    {
        FSDK_OutputLog("time is %ds and %dus\n",tv_end.tv_sec - tv_start.tv_sec, tv_end.tv_usec - tv_start.tv_usec);
    }
    else
    {
        FSDK_OutputLog("time is %ds and %dus\n",tv_end.tv_sec - tv_start.tv_sec - 1, tv_end.tv_usec + 1000000 - tv_start.tv_usec);
    }

	//Close log file.
	FSDK_CloseLog();

	return 0;
}

