/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is an example to implement generating barcode and saving it as an image file.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main barcode
#endif

/*******************************************************************************/
/* Macro definitions                                                           */
/*                                                                             */
/* Barcode formats type string which to be generated barcode.				   */
/*******************************************************************************/

#define    CODE_39      "TEST-SHEET"           //CODE_39 type string
#define    CODE_128     "102030405060708090"   //CODE_128 type string
#define    EAN_8        "80674313"             //EAN_8 type string
#define    UPC_A        "890444000335"         //UPC_A type string
#define    EAN_13       "9780804816632"        //EAN_13 type string
#define    ITF          "070429"               //ITF type string
#define    PDF_417      "Unknown - change me!" //PDF_417 type string
#define    QR_CODE      "TestForBarcodeQrCode" //QR_CODE type string

//Generating each type barcode and saving it as an image file.
void Barcode_GenerateEachType(string codeStr, FS_INT32 codeFormat, FS_INT32 unitWidth, FS_INT32 unitHeight, 
								FS_INT32 qrLevel, string bmpName)
{
	//Get path of output file folder for this demo.
	string outputFilesPath = FSDK_GetOutputFilesFolder("barcode");
	//Get the string which to be generated barcode.
	FSCRT_BSTR info = {(FS_LPSTR)codeStr.c_str(), (FS_DWORD)codeStr.length()};

	//Generate barcode
	
	CAutoImage autoImage;
	FS_RESULT ret =	autoImage.GenerateBitmapbyBarcode(&info, codeFormat, unitWidth, unitHeight, qrLevel);

	if (FSCRT_ERRCODE_SUCCESS != ret) 
		return;

	//Save barcode as an image file
	FSDK_SaveImageFile(autoImage.GetBitmap(), FSCRT_IMAGETYPE_BMP, (FS_LPSTR)(outputFilesPath + bmpName).c_str());
}

//Generating all type barcodes and saving them as image files.
void Barcode_GenerateAll( )
{
	//log information 
	string logBarcodeStr[8] = {"\r\nGenerate 'TEST-SHEET' CODE_39 Bitmap: ", 
						"\r\nGenerate '102030405060708090' CODE_128 Bitmap: ",
						"\r\nGenerate '80674313' EAN_8 Bitmap: ", 
						"\r\nGenerate '890444000335' UPC_A Bitmap: ", 
						"\r\nGenerate '9780804816632' EAN_13 Bitmap: ", 
						"\r\nGenerate '070429' ITF Bitmap: ", 
						"\r\nGenerate 'Unknown - change me!' PDF_417 Bitmap: ", 
						"\r\nGenerate 'TestForBarcodeQrCode' QR_CODE Bitmap: "};

	//Barcode type string which to be generated barcode.	
	string codeStr[8] = {CODE_39, CODE_128, EAN_8, UPC_A, EAN_13, ITF, PDF_417, QR_CODE};
	//Barcode formats type
	FS_INT32 codeFormat[8] = {FSCRT_BARCODEFORMAT_CODE_39, FSCRT_BARCODEFORMAT_CODE_128,
							  FSCRT_BARCODEFORMAT_EAN_8, FSCRT_BARCODEFORMAT_UPC_A, 
						      FSCRT_BARCODEFORMAT_EAN_13, FSCRT_BARCODEFORMAT_ITF, 
		                      FSCRT_BARCODEFORMAT_PDF_417, FSCRT_BARCODEFORMAT_QR_CODE};

	//Image name which the generated barcodes are to be saved as.(except QR_CODE)
	string bmpName[7] = {"/code39_TEST-SHEET.bmp", "/CODE_128_102030405060708090.bmp",
						 "/EAN_8_80674313.bmp", "/UPC_A_890444000335.bmp", "/EAN_13_9780804816632.bmp",
						 "/ITF_070429.bmp", "/PDF_417_Unknown.bmp"};

	//Format error correction level of QR code.
	FS_INT32 qrLevel[4] = {FSCRT_BARCODE_QRERRORCORRECTION_L, FSCRT_BARCODE_QRERRORCORRECTION_M, 
						   FSCRT_BARCODE_QRERRORCORRECTION_Q, FSCRT_BARCODE_QRERRORCORRECTION_H};
	//Image name which the generated QR barcodes are to be saved as.
	string bmpQrName[4] = {"/QR_CODE_TestForBarcodeQrCode_L.bmp", "/QR_CODE_TestForBarcodeQrCode_M.bmp",
						   "/QR_CODE_TestForBarcodeQrCode_Q.bmp", "/QR_CODE_TestForBarcodeQrCode_H.bmp"};

	FS_INT32 unitWidth = 2;//Unit width for barcode in pixels, preferred value is 1-5 pixels.
	FS_INT32 unitHeight = 120;//Unit height for barcode in pixels, preferred value is >= 20 pixels.

	//Generate each barcode formats type
	for (int i = 0; i < 8; i++)
	{
		FSDK_OutputLog("%s\r\n",logBarcodeStr[i].c_str());
		if (codeFormat[i] != FSCRT_BARCODEFORMAT_QR_CODE)//if not QR_CODE formats type
			Barcode_GenerateEachType(codeStr[i], codeFormat[i], unitWidth, unitHeight, 0, bmpName[i]);
		else//QR_CODE formats type
		{
			for (int j = 0; j < 4; j++)//Generate QR_CODE formats barcode for each Format error correction level.
				Barcode_GenerateEachType(codeStr[i], codeFormat[i], unitWidth, unitHeight, qrLevel[j], bmpQrName[j]);
		}
	}
	
}

//Entrance of this example
int main(int argc, char* argv[])
{

	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("barcode");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: barcode\r\n\r\n");


	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Failed to initialize, output status and return
		FSDK_CloseLog();//Close folder
		return -1;
	}

	//Initialize barcode module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FS_RESULT ret = FSCRT_BCModule_Initialize();
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_OutputLog("Failure: Fail to initialize BCModule and the error code %d.\r\n", ret);
		NO_LICENSE("Barcode"); 
		FSDK_CloseLog();
		return -1;
	}


	//Generating barcodes
	Barcode_GenerateAll();

	//Finalize barcode module
	FSCRT_BCModule_Finalize();

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();

	//Finalize Foxit SDK library
	FSDK_FinalizeLibrary();

	//Close log and exit
	FSDK_CloseLog();
	return 0;
}
