/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to implement functions of importing form from FDF, exporting form to FDF and get information of form fields.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfforminfo
#endif

class CAutoForm
{
public:
	CAutoForm();
	~CAutoForm();

	FS_RESULT Init(FSCRT_DOCUMENT pdfDoc);
	FS_RESULT ImportFromFDFDoc(FSCRT_DOCUMENT fdfDoc);
	FS_RESULT ExportToFDFDoc(FSCRT_DOCUMENT fdfDoc);
	FS_RESULT CountFields(FSCRT_BSTR* filter, FS_INT32* count);
	FS_RESULT GetField(FSCRT_BSTR* filter, FS_INT32 index, FSCRT_BSTR* fieldName, FS_INT32* fieldType);
	FS_RESULT GetAlignment(FSCRT_BSTR* fieldName, FS_INT32* alignment);
	FS_RESULT GetValue(FSCRT_BSTR* fieldName, FSCRT_BSTR* value);
	FS_RESULT GetDefaultValue(FSCRT_BSTR* fieldName, FSCRT_BSTR* value);
	FS_RESULT CountFieldControls(FSCRT_PAGE page, FSCRT_BSTR* fieldName, FS_INT32* count);

	FSPDF_FORM m_form;
};

CAutoForm::CAutoForm()
	:m_form(NULL)
{
}

CAutoForm::~CAutoForm()
{
	if (m_form)
	{
		FSPDF_Form_Release(m_form);
		m_form = NULL;
	}
}

FS_RESULT CAutoForm::Init(FSCRT_DOCUMENT pdfDoc)
{
	FS_RESULT ret = FSPDF_Form_Load(pdfDoc, &m_form);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Form_Load with error code %d.\r\n", ret);
		FSDK_CHECK_LICENSE(ret);
	}
	else
		FSDK_OutputLog("Success: Load PDF form.\r\n");
}

FS_RESULT CAutoForm::ImportFromFDFDoc(FSCRT_DOCUMENT fdfDoc)
{
	FS_RESULT ret = FSPDF_Form_ImportFromFDFDoc(m_form, fdfDoc);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_Form_ImportFromFDFDoc with error code %d.\r\n", ret);
	else
		FSDK_OutputLog("Success: Import from FDF doc.\r\n");
	return ret;
}

FS_RESULT CAutoForm::ExportToFDFDoc(FSCRT_DOCUMENT fdfDoc)
{
	FS_RESULT ret = FSPDF_Form_ExportToFDFDoc(m_form, fdfDoc);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_Form_ExportToFDFDoc with error code %d.\r\n", ret);
	else
		FSDK_OutputLog("Success: Export to FDF doc.\r\n");
	return ret;
}

FS_RESULT CAutoForm::CountFields(FSCRT_BSTR* filter, FS_INT32* count)
{
	FS_RESULT ret = FSPDF_Form_CountFields(m_form, filter, count);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: Fail to get the count of fields with filter \"%s\", with error code %d.\r\n", filter->str, ret);

	return ret;
}

FS_RESULT CAutoForm::GetField(FSCRT_BSTR* filter, FS_INT32 index, FSCRT_BSTR* fieldName, FS_INT32* fieldType)
{
	FS_RESULT ret = FSPDF_Form_GetField(m_form, filter, index, fieldName, fieldType);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_Form_GetField with error code %d.\r\n", ret);

	return ret;
}

FS_RESULT CAutoForm::GetAlignment(FSCRT_BSTR* fieldName, FS_INT32* alignment)
{
	FS_RESULT ret = FSPDF_FormField_GetAlignment(m_form, fieldName, alignment);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_FormField_GetAlignment with error code %d.\r\n", ret);

	return ret;
}

FS_RESULT CAutoForm::GetValue(FSCRT_BSTR* fieldName, FSCRT_BSTR* value)
{
	FS_RESULT ret = FSPDF_FormField_GetValue(m_form, fieldName, value);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_FormField_GetValue with error code %d.\r\n", ret);

	return ret;
}

FS_RESULT CAutoForm::GetDefaultValue(FSCRT_BSTR* fieldName, FSCRT_BSTR* value)
{
	FS_RESULT ret = FSPDF_FormField_GetDefaultValue(m_form, fieldName, value);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_FormField_GetDefaultValue with error code %d.\r\n", ret);

	return ret;
}

FS_RESULT CAutoForm::CountFieldControls(FSCRT_PAGE page, FSCRT_BSTR* fieldName, FS_INT32* count)
{
	FS_RESULT ret = FSPDF_FormField_CountControls(m_form, page, fieldName, count);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: In function FSPDF_FormField_CountControls with error code %d.\r\n", ret);

	return ret;
}

//Import form to PDF from FDF
void ImportFormFDF(string fdfName, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Import FDF From Form ======\r\n");
	CAutoDocument fdfDoc((inputFilesPath + "/" + fdfName).c_str(), "rb");
	if (!fdfDoc.GetFileHander())
		return;

	if (!fdfDoc.LoadFDFDocument())
		return;

	FSCRT_BSTR pdfpath;
	FSCRT_BStr_Init(&pdfpath);
	//Get PDF path from FDF file
	FS_RESULT ret = FSFDF_Doc_GetPDFPath(fdfDoc.GetDocument(), &pdfpath);
	string strFile = string(pdfpath.str, pdfpath.len);
	FSCRT_BStr_Clear(&pdfpath);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSFDF_Doc_GetPDFPath with error code %d.\r\n", ret);
		return;
	}

	CAutoDocument pdfDoc((inputFilesPath + "/" + strFile).c_str(), "rb");
	if (!pdfDoc.GetFileHander())
		return;

	if (FSCRT_ERRCODE_SUCCESS != pdfDoc.LoadPDFDocument(NULL))
		return;

	CAutoForm pdfForm;
	pdfForm.Init(pdfDoc.GetDocument());
	if (!pdfForm.m_form)
		return;

	//Import form to PDF from FDF
	pdfForm.ImportFromFDFDoc(fdfDoc.GetDocument());

	//Save PDF to output files folder
	string pdfName = string(fdfName.c_str(), fdfName.length() - 4 );
	pdfName += ".pdf";
	FSDK_SavePDFFile(pdfDoc.GetDocument(), (outputFilesPath + "/" + pdfName).c_str());
}

//Get form field information and export form to FDF from PDF
void GetFormInfoAndExportFDF(string pdfName, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Get Form Info And Export FDF ======\r\n");
	CAutoDocument pdfDoc((inputFilesPath + "/" + pdfName).c_str(), "rb");
	if (!pdfDoc.GetFileHander())
		return;

	if (FSCRT_ERRCODE_SUCCESS != pdfDoc.LoadPDFDocument(NULL))
		return;

	CAutoForm pdfForm;
	pdfForm.Init(pdfDoc.GetDocument());
	if (!pdfForm.m_form)
		return;

	//Create FDF
	FSCRT_DOCUMENT fdfDoc = NULL;
	FS_RESULT ret = FSFDF_Doc_Create(FSCRT_DOCUMENTTYPE_FDF, &fdfDoc);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSFDF_Doc_Create with error code %d.\r\n", ret);
		return;
	}

	//Export form to FDF from PDF
	pdfForm.ExportToFDFDoc(fdfDoc);

	string fdfName = string(pdfName.c_str(), pdfName.length() - 4);
	fdfName += ".fdf";
	FSCRT_FILE saveFile = FSDK_OpenFile((outputFilesPath + "/" + fdfName).c_str(), "wb");
	if (saveFile)
	{
		//Save FDF document
		ret = FSFDF_Doc_Save(fdfDoc, saveFile);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			FSDK_OutputLog("Failure: In function FSFDF_Doc_Save with error code %d.\r\n", ret);
		else{
			string fdfFileName = FSDK_GetFileName(fdfName.c_str());
			FSDK_OutputLog("Success: The FDF document save to %s.\r\n", fdfFileName.c_str());
		}
		FSCRT_File_Release(saveFile);
	}

	//Close FDF document
	FSFDF_Doc_Close(fdfDoc);
	
	FSDK_OutputLog("\r\n--- Form information ---\r\n");
	//Count fields
	FS_INT32 nFieldCount = 0;
	ret = pdfForm.CountFields(NULL, &nFieldCount);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		FSDK_OutputLog("There are %d filed(s).\r\n", nFieldCount);
	for (int i = 0; i < nFieldCount; i++)
	{
		FSCRT_BSTR strName;
		FSCRT_BStr_Init(&strName);
		FS_INT32 nType = 0;
		FSDK_OutputLog("\r\nThe field with index %d Information:\r\n", i);
		//Get name and type of form field by index
		ret = pdfForm.GetField(NULL, i, &strName, &nType);
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			FSDK_OutputLog("Name: %s.\r\n", strName.str);
			switch (nType)
			{
			case FSPDF_FORMFIELDTYPE_UNKNOWN:
				FSDK_OutputLog("Type: Unknown.\r\n");
				break;
			case FSPDF_FORMFIELDTYPE_PUSHBUTTON:
				FSDK_OutputLog("Type: Push Button.\r\n");
				break;
			case FSPDF_FORMFIELDTYPE_CHECKBOX:
				FSDK_OutputLog("Type: Check Box.\r\n");
				break;
			case FSPDF_FORMFIELDTYPE_RADIOBUTTON:
				FSDK_OutputLog("Type: Radio Button.\r\n");
				break;
			case FSPDF_FORMFIELDTYPE_COMBOBOX:
				FSDK_OutputLog("Type: Combo Box.\r\n");
				break;
			case FSPDF_FORMFIELDTYPE_LISTBOX:
				FSDK_OutputLog("Type: List Box.\r\n");
				break;
			case FSPDF_FORMFIELDTYPE_TEXTFIELD:
				FSDK_OutputLog("Type: Text Field.\r\n");
				break;
			}
		}
		//Get alignment of form field by name
		FS_INT32 nAliment = -1;
		ret = pdfForm.GetAlignment(&strName, &nAliment);	//Get alignment
		if (FSCRT_ERRCODE_SUCCESS == ret){
			switch (nAliment)
			{
			case 0:
				FSDK_OutputLog("Alignment: Left.\r\n");
				break;
			case 1:
				FSDK_OutputLog("Alignment: Centered.\r\n");
				break;
			case 2:
				FSDK_OutputLog("Alignment: Right.\r\n");
				break;
			}
		}

		if (nType != FSPDF_FORMFIELDTYPE_PUSHBUTTON)		//There are not value and default value in push button
		{
			//Get value of form field by name
			FSCRT_BSTR strVal;
			FSCRT_BStr_Init(&strVal);
			ret = pdfForm.GetValue(&strName, &strVal);		//Get value
			if (FSCRT_ERRCODE_SUCCESS == ret)
				FSDK_OutputLog("Value: %s.\r\n", strVal.str);
			FSCRT_BStr_Clear(&strVal);

			//Get default value of form field by name
			FSCRT_BStr_Init(&strVal);
			ret = pdfForm.GetDefaultValue(&strName, &strVal);	//Get default value
			if (FSCRT_ERRCODE_SUCCESS == ret)
				FSDK_OutputLog("Default Value: %s.\r\n", strVal.str);
			FSCRT_BStr_Clear(&strVal);
		}

		FS_INT32 nPageCount = pdfDoc.GetPagesCount();
		FS_INT32 nContrCount = 0;
		for (int i = 0; i < nPageCount; i++)
		{
			CAutoPDFPage pdfPage;
			ret = pdfPage.LoadPDFPage(pdfDoc.GetDocument(), i);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSCRT_BStr_Clear(&strName);
				continue;
			}
			pdfPage.LoadAnnots();			//Must load annots before count controls
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSCRT_BStr_Clear(&strName);
				continue;
			}

			//Count controls of form field by name
			FS_INT32 nCount = 0;
			pdfForm.CountFieldControls(pdfPage.GetPDFPage(), &strName, &nCount);
			nContrCount += nCount;
		}
		FSDK_OutputLog("Controls' Count: %d\r\n", nContrCount);

		FSCRT_BStr_Clear(&strName);
	}
}

void FormFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{//Enumerate files in input file folder and choose PDF or FDF file to load
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".fdf"))
			{
				//Import form from FDF to PDF
				ImportFormFDF(strFile, inputFilesPath, outputFilesPath);
			}

			//Get the name of each PDF file
			if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
			{
				//Get information from form fields and export form to FDF from PDF
				GetFormInfoAndExportFDF(strFile, inputFilesPath, outputFilesPath);
			}
		}

		if (strFile.size() < 1) break;
	} while (1);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	string logPath = FSDK_GetLogFile("pdfforminfo");
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfforminfo\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfforminfo\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get output files folder path.
	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfforminfo");
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get path of input file folder for this demo
	string inputFilesPath = FSDK_GetInputFilesFolder("pdfforminfo");
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());

	if (folder)
	{
		FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfforminfo: ");

		FormFolder(folder, inputFilesPath, outputFilesPath);
		//Close access to specific folder
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();

		string fdfName = "InformationCollectionData.fdf";
		//import forminfo from fdf document
		ImportFormFDF(fdfName, inputFilesPath, outputFilesPath);

		string pdfName = "InformationCollection.pdf";
		//export fdf information from PDF file
		GetFormInfoAndExportFDF(pdfName, inputFilesPath, outputFilesPath);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}