/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to get catalog dictionary from a PDF file and set some properties of catalog dictionary.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfobjects
#endif

FS_RESULT SetCatalogDict(FSCRT_DOCUMENT pdfDoc)
{//This function is to set some properties of catalog dictionary.

	FSPDF_OBJECT catalogObj = NULL;
	//Get catalog dictionary.
	FS_RESULT ret = FSPDF_Doc_GetCatalog(pdfDoc, &catalogObj);
	if (FSCRT_ERRCODE_SUCCESS == ret && catalogObj)
	{
		FSCRT_BSTR bstrKey[2];
		FSCRT_BSTR bstrValueToSet[2];
		FSCRT_BSTR bstrValueToSet2[2];
		FSCRT_BSTR bstrValue;
		//Set key and values.
		FSCRT_BStr_InitConstString(bstrKey[0], "PageLayout");
		FSCRT_BStr_InitConstString(bstrValueToSet[0], "TwoColumnRight");
		FSCRT_BStr_InitConstString(bstrValueToSet2[0], "SinglePage");
		FSCRT_BStr_InitConstString(bstrKey[1], "PageMode");
		FSCRT_BStr_InitConstString(bstrValueToSet[1], "FullScreen");
		FSCRT_BStr_InitConstString(bstrValueToSet2[1], "UseNone");
		
		for (int i=0; i<2; i++)
		{
			//Initialize FSCRT_BSTR object before use it.
			FSCRT_BStr_Init(&bstrValue);
			//Get value of specific property.
			ret = FSPDF_Dictionary_GetUnicodeString(pdfDoc, catalogObj, &bstrKey[i], &bstrValue);
			if (FSCRT_ERRCODE_NOTFOUND == ret)
			{//If not found, just add property with value bstrValueToSet[i].
				FSPDF_Dictionary_SetAtUnicodeName(pdfDoc, catalogObj, &bstrKey[i], &bstrValueToSet[i]);
			}
			else if (FSCRT_ERRCODE_SUCCESS == ret)
			{//If property exists, change its value.
				if (0 == strcmp(bstrValue.str, bstrValueToSet[i].str))
				{//If Old value equals bstrValueToSet[i], change to use bstrValueToSet2[i] to set value.
					FSPDF_Dictionary_SetAtUnicodeName(pdfDoc, catalogObj, &bstrKey[i], &bstrValueToSet2[i]);
				}
				else
				{//Set property's value to bstrValueToSet[i].
					FSPDF_Dictionary_SetAtUnicodeName(pdfDoc, catalogObj, &bstrKey[i], &bstrValueToSet[i]);
				}
			}
			else
			{
				//Release FSCRT_BSTR object.
				FSCRT_BStr_Clear(&bstrValue);
				return FSCRT_ERRCODE_ERROR;
			}
			//Release FSCRT_BSTR object.
			FSCRT_BStr_Clear(&bstrValue);
		}

		return FSCRT_ERRCODE_SUCCESS;
	}
	else if (FSCRT_ERRCODE_SUCCESS == ret && NULL == catalogObj)
		return FSCRT_ERRCODE_NOTFOUND;
	else
	{
		FSDK_CHECK_LICENSE(ret);
		return FSCRT_ERRCODE_ERROR;
	}
}

FS_RESULT	LoadPDFFile(string fileName, string inputFolder)
{//This function is to load PDF document. If load a PDF page successfully, set some properties of catalog dictionary.
	string strLog("\r\n======");
	//Get path of input files folder of this demo.
	string strFile = inputFolder;
	//Set path of input file.
	strFile += "/";
	strFile += fileName;
	strLog += fileName;

	string strFilename = string(fileName.c_str(), (fileName.length()-4));
	
	strLog += "\r\n\topen file: ";
	//Open input file and create file handle.
	FSCRT_FILE file = FSDK_OpenFile(strFile.c_str(), "rb");
	if (!file)
	{
		strLog += "failure\r\n";
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		return FSCRT_ERRCODE_ERROR;
	}
	strLog += "success\r\n";

	strLog += "\tload PDF document: ";
	FSCRT_DOCUMENT pdfDoc = NULL;
	//Load PDF document handle.
	FS_RESULT ret = FSPDF_Doc_StartLoad(file, NULL, &pdfDoc, NULL);
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		strLog += "success\r\n";

		strLog += "\tset catalog dictionary: ";
		//Set catalog dictionary: properties "PageLayout" and "PageMode"
		ret = SetCatalogDict(pdfDoc);
		switch (ret)
		{
		case FSCRT_ERRCODE_SUCCESS:
			{
				strLog += "success\r\n";
				//Get output file folder
				string strOutfile = FSDK_GetOutputFilesFolder("pdfobjects");
				//Set output PDF file name
				strOutfile += "/" + strFilename + "_changedDict.pdf";
				strLog += "\tsave to \"" + strFilename + "_changedDict.pdf\": ";
				//Save to specific file, saving mode: no original.
				FS_BOOL bRet = FSDK_SavePDFFile(pdfDoc, strOutfile.c_str(), FSPDF_SAVEFLAG_NOORIGINAL);
				if (bRet)
					strLog += "success\r\n";
				else
					strLog += "failure\r\n";
				break;
			}
		case FSCRT_ERRCODE_NOTFOUND:
			{
				strLog += "no catalog dictionary in this PDF file.\r\n";
				break;
			}
		case FSCRT_ERRCODE_ERROR:
		default:
			{
				strLog += "failure\r\n";
				break;
			}
		}
		//Close PDF document handle.
		FSPDF_Doc_Close(pdfDoc);
		pdfDoc = NULL;
	}
	else
		strLog += "failure\r\n";
	//Release file handle.
	FSCRT_File_Release(file);
	//Output log string to log.txt in output file folder.
	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
	
	return ret;
}

// get object from pdf file
void GetObjectFromPDFFile(string pdfFile, string inputFilesPath, string outputFilesPath )
{
	if (pdfFile.size() > 0 && FSDK_CompareExtName(pdfFile.c_str(), ".pdf"))
	{
		FS_RESULT nRet= LoadPDFFile(pdfFile ,inputFilesPath);
		if (nRet == FSCRT_ERRCODE_SUCCESS)
			FSDK_OutputLog("Succeed to get object from PDF file\r\n");
		else
			FSDK_OutputLog("Failed to get object from PDF file\r\n");
	}
}

void  BrowerFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	do 
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			GetObjectFromPDFFile(strFile, inputFilesPath, outputFilesPath);
		}
		if (strFile.size() < 1) break;

	} while (1);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);

	string logPath = FSDK_GetLogFile("pdfobjects");
	//Open log file.
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfobjects\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfobjects\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_OutputLog("failure\r\n");
		FSDK_CloseLog();
		return -1;
	}
	FSDK_OutputLog("success\r\n");

	//Initialize PDF module, or no method of PDF module can be used.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfobjects");
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get path of input file folder for this demo.
	string inputFilesPath = FSDK_GetInputFilesFolder("pdfobjects");

	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());
	if (folder)
	{
		FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfobjects: ");
		BrowerFolder(folder, inputFilesPath, outputFilesPath);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		string pdffile = "AboutFoxit.pdf";
		GetObjectFromPDFFile(pdffile, inputFilesPath, outputFilesPath);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
