/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to search default pattern "Foxit" in PDF file and save the search result to a .txt file.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfsearch
#endif

class CAutoTextSearch
{
public:
	CAutoTextSearch(FSPDF_TEXTPAGE textPage, const FSCRT_BSTR* searchPattern, FS_DWORD flags, FS_INT32 startIndex);
	~CAutoTextSearch();

	FS_RESULT FindNext(FS_BOOL* isMatch);
	
	FSPDF_TEXTSEARCH m_textSearch;
	
};

class CAutoTextSelection
{
public:
	CAutoTextSelection(FSPDF_TEXTSEARCH textSearch);
	~CAutoTextSelection();

	FS_RESULT CountPieces(FS_INT32* count);
	FS_RESULT GetPieceCharRange(FS_INT32 pieceIndex, FS_INT32* start, FS_INT32* count);
	FS_RESULT GetPieceRect(FS_INT32 pieceIndex, FSCRT_RECTF* rect);

	FSPDF_TEXTSELECTION m_textSelection;
};

CAutoTextSearch::CAutoTextSearch(FSPDF_TEXTPAGE textPage, const FSCRT_BSTR* searchPattern, FS_DWORD flags, FS_INT32 startIndex)
	:m_textSearch(NULL)
{
	FS_RESULT ret = FSPDF_TextPage_StartSearch(textPage, searchPattern, flags, startIndex, &m_textSearch);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure:In function FSPDF_TextPage_StartSearch with error code %d.\r\n", ret);
}

CAutoTextSearch::~CAutoTextSearch()
{
	if (m_textSearch)
	{
		FSPDF_TextSearch_Release(m_textSearch);
		m_textSearch = NULL;
	}
}

FS_RESULT CAutoTextSearch::FindNext(FS_BOOL* isMatch)
{
	FS_RESULT ret = FSPDF_TextSearch_FindNext(m_textSearch, isMatch);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure:In function FSPDF_TextSearch_FindNext with error code %d.\r\n", ret);

	return ret;
}

CAutoTextSelection::CAutoTextSelection(FSPDF_TEXTSEARCH textSearch)
	:m_textSelection(NULL)
{
	FS_RESULT ret = FSPDF_TextSearch_GetSelection(textSearch, &m_textSelection);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure:In function FSPDF_TextSearch_GetSelection with error code %d.\r\n", ret);
}

CAutoTextSelection::~CAutoTextSelection()
{
	if (m_textSelection)
	{
		FSPDF_TextSelection_Release(m_textSelection);
		m_textSelection = NULL;
	}
}

FS_RESULT CAutoTextSelection::CountPieces(FS_INT32* count)
{
	FS_RESULT ret = FSPDF_TextSelection_CountPieces(m_textSelection, count);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure:In function FSPDF_TextSelection_CountPieces with error code %d.\r\n", ret);

	return ret;
}

FS_RESULT CAutoTextSelection::GetPieceCharRange(FS_INT32 pieceIndex, FS_INT32* start, FS_INT32* count)
{
	FS_RESULT ret = FSPDF_TextSelection_GetPieceCharRange(m_textSelection, pieceIndex, start, count);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure:In function FSPDF_TextSelection_GetPieceCharRange with error code %d.\r\n", ret);

	return ret;
}

FS_RESULT CAutoTextSelection::GetPieceRect(FS_INT32 pieceIndex, FSCRT_RECTF* rect)
{
	FS_RESULT ret = FSPDF_TextSelection_GetPieceRect(m_textSelection, pieceIndex, rect);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure:In function FSPDF_TextSelection_CountPieces with error code %d.\r\n", ret);

	return ret;
}

//Do the search process in a PDF page.
void Search(string pdfFile, string inputFilesPath, string searchText)
{
	CAutoDocument pdfDoc((inputFilesPath + "/" + pdfFile).c_str(), "rb");
	if (!pdfDoc.GetFileHander())
		return;

	if (FSCRT_ERRCODE_SUCCESS != pdfDoc.LoadPDFDocument(NULL))
		return;

	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	FS_INT32 pageCount = pdfDoc.GetPagesCount();
	for (int i = 0; i < pageCount; i++)
	{
		CAutoPDFPage pdfPage;
		ret = pdfPage.LoadPDFPage(pdfDoc.GetDocument(), i);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			return;

		ret = pdfPage.ParsePage();
		if (FSCRT_ERRCODE_SUCCESS != ret)
			return;

		ret = pdfPage.LoadTextPage();
		if (FSCRT_ERRCODE_SUCCESS != ret)
			return;

		//Set searchPattern
		FSCRT_BSTR searchPattern;
		//Initialize const string to searchPattern.
		FSCRT_BStr_Init(&searchPattern);
		FSCRT_BStr_Set(&searchPattern, (char*)searchText.c_str(), (FS_DWORD)searchText.length());

		FSDK_OutputLog("Start to search \"%s\":\r\n", searchText.c_str());
		CAutoTextSearch textSearch(pdfPage.GetPDFTextPage(), &searchPattern, 0, 0);
		if (!textSearch.m_textSearch)
		{
			FSCRT_BStr_Clear(&searchPattern);
			return;
		}
		//Find next matched pattern, direction from page top to down. 
		//Here, the first match will be found if exists and searching process starts from here.
		FS_BOOL isMatch = FALSE;
		textSearch.FindNext(&isMatch);
		FS_INT32 iMatchCount = 0;//Used to record match count.
		while (isMatch)
		{
			//If find one match, get its character range and rectangle of this match in PDF space.
			iMatchCount++;
			FSDK_OutputLog("Match time: %d\r\n", iMatchCount);
			
			//Get the text selection handle.
			CAutoTextSelection textSelection(textSearch.m_textSearch);
			if (!textSelection.m_textSelection)
			{
				FSCRT_BStr_Clear(&searchPattern);
				return;
			}
			//Count pieces within text selection.
			FS_INT32 iPieceCount = 0;
			textSelection.CountPieces(&iPieceCount);
			//Get some information one piece by one piece.
			for (FS_INT32 i=0; i<iPieceCount; i++)
			{
				FS_INT32 iStartChar = 0;
				FS_INT32 iCountChar = 0;
				FSCRT_RECTF selRect;
				//Get char range of specific piece: index of start character and cout of characters in a piece.
				textSelection.GetPieceCharRange(i, &iStartChar, &iCountChar);
				//Get rectangle of specific piece.
				textSelection.GetPieceRect(i, &selRect);
				FSDK_OutputLog("\tStart character index: %d\r\n\tRectangle(in PDF space):\r\n\tLeft:%.2f Top:%.2f Right:%.2f Bottom:%.2f\r\n", iStartChar, selRect.left, selRect.top, selRect.right, selRect.bottom);
			}

			//Find the next match. If no match, this loop will be finished.
			textSearch.FindNext(&isMatch);
		}

		if (0 == iMatchCount)
			FSDK_OutputLog("\tNo match pattern.\r\n");
		else
			FSDK_OutputLog("All match count: %d\r\n\r\n", iMatchCount);
		FSCRT_BStr_Clear(&searchPattern);
	}

	FSDK_OutputLog("Succeed to search.\r\n");
}

void  BrowerFolder(void* folder, string inputFilesPath, string outputFilesPath, string searchText)
{
	do 
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir && strFile.size() > 0 && FSDK_CompareExtName(strFile.c_str(), ".pdf"))
		{
			Search(strFile, inputFilesPath, searchText);
		}
		if (strFile.size() < 1) break;

	} while (1);
}


int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);
	string logPath = FSDK_GetLogFile("pdfsearch");
	//Open log file.
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfsearch\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfsearch\r\n\r\n");

	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfsearch");
	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get path of input file folder for this demo.
	string inputFilesPath = FSDK_GetInputFilesFolder("pdfsearch");
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str(), FALSE);
	if (folder)
	{
		FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfsearch: ");
		BrowerFolder(folder, inputFilesPath, outputFilesPath, "Foxit");
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		string pdffile = "AboutFoxit.pdf";
		Search(pdffile, inputFilesPath, "Foxit");
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
