/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This demo illustrates how  to use  Foxit  PDF  SDK  to add  different types of watermarks into PDF files. 
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfwatermark
#endif

class CAutoWatermark
{
public:
	CAutoWatermark();
	CAutoWatermark(const char* fileName, const char* fileMode);
	~CAutoWatermark();

	FS_RESULT CreateFromText(FSCRT_BSTR* text, FSPDF_WATERMARK_TEXTPROPERTIES* properties);
	FS_RESULT CreateFromImage(FSCRT_IMAGE image);
	FS_RESULT CreateFromPage(FSCRT_PAGE page);
	FS_RESULT InsertToPage();

	CAutoDocument m_doc;
	FSPDF_WATERMARK_SETTINGS m_setting;
	FSPDF_WATERMARK m_watermark;
};

CAutoWatermark::CAutoWatermark(const char* fileName, const char* fileMode)
	:m_doc(fileName, fileMode)
	,m_watermark(NULL)
{
	if (m_doc.GetFileHander())
		m_doc.LoadPDFDocument(NULL);

	m_setting.position = FSPDF_WATERMARKPOS_CENTER;
	m_setting.offsetX = 0;
	m_setting.offsetY = 0;
	m_setting.flags = FSPDF_WATERMARKFLAG_ONTOP;
	m_setting.scaleX = 0.25f;
	m_setting.scaleY = 0.25f;
	m_setting.rotation = 0;
	m_setting.opacity = 100;
}

CAutoWatermark::~CAutoWatermark()
{
	if (m_watermark)
		FSPDF_Watermark_Release(m_watermark);
}

FS_RESULT CAutoWatermark::CreateFromText(FSCRT_BSTR* text, FSPDF_WATERMARK_TEXTPROPERTIES* properties)
{
	return FSPDF_Watermark_CreateFromText(m_doc.GetDocument(), text, properties, &m_setting, &m_watermark);
}

FS_RESULT CAutoWatermark::CreateFromImage(FSCRT_IMAGE image)
{
	return FSPDF_Watermark_CreateFromImage(m_doc.GetDocument(), image, &m_setting, &m_watermark);
}

FS_RESULT CAutoWatermark::CreateFromPage(FSCRT_PAGE page)
{
	return FSPDF_Watermark_CreateFromPage(m_doc.GetDocument(), page, &m_setting, &m_watermark);
}

FS_RESULT CAutoWatermark::InsertToPage()
{
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;

	int page_count = 0;
	ret = FSPDF_Doc_CountPages(m_doc.GetDocument(), &page_count);
	for (int i = 0; i < page_count; i++)
	{
		CAutoPDFPage autoPDFPage;
		ret = autoPDFPage.LoadPDFPage(m_doc.GetDocument(), i);
		if (FSCRT_ERRCODE_SUCCESS != ret)
			return ret;

		ret = autoPDFPage.ParsePage();
		if (FSCRT_ERRCODE_SUCCESS != ret)
			return ret;

		ret = FSPDF_Watermark_InsertToPage(m_watermark, autoPDFPage.GetPDFPage());
		if (ret != FSCRT_ERRCODE_SUCCESS)
		{
			FSDK_OutputLog("Failure: In function FSPDF_Watermark_InsertToPage with error code %d.\r\n", ret); 
			return ret;
		}
	}

	return ret;
}

FS_RESULT InsertFilewatermark(string strFile, string inputFilesPath, string outputFile)
{
	int pos = (int)strFile.find_first_of('.');
	string strFileName = strFile;
	strFileName = strFileName.erase(pos, 4);
	string strLog;
	//Do different kind of watermark.
	for (int icount = 0; icount < 3; icount++)
	{
		CAutoWatermark autoWatermark((inputFilesPath + "/" + strFile).c_str(), "rb");
		if (!autoWatermark.m_doc.GetDocument())
			return FSCRT_ERRCODE_ERROR;

		FS_RESULT ret = FSCRT_ERRCODE_ERROR;
		//Text watermark
		if (icount == 0)
		{
			FSDK_OutputLog("Begin to add text watermark.\r\n");
			//Init textproperties.
			FSPDF_WATERMARK_TEXTPROPERTIES textproperties;
			textproperties.font = NULL;
			ret = FSCRT_Font_CreateStandard(FSCRT_STDFONT_HELVETICA, &textproperties.font);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSCRT_Font_CreateStandard with error code %d.\r\n", ret); 
				return ret;
			}
			textproperties.fontSize = 100.0f;
			textproperties.color = FSCRT_ARGB_Encode(0xff, 0xff, 0x00, 0x00);
			textproperties.fontStyle = FSPDF_WATERMARK_FONTSTYLE_UNDERLINE;
			textproperties.lineSpace = 1;
			textproperties.alignment = FSPDF_WATERMARK_TEXTALIGNMENT_CENTER;

			FSCRT_BSTRC(unicodeString, "Hello!");
			//Insert text to document.
			ret = autoWatermark.CreateFromText(&unicodeString, &textproperties);
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Watermark_CreateFromText with error code %d.\r\n", ret); 
				FSCRT_Font_Release(textproperties.font);
				FSDK_CHECK_LICENSE(ret);
				return ret;
			}

			ret = autoWatermark.InsertToPage();
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSCRT_Font_Release(textproperties.font);
				return ret;
			}
			//Release font.
			FSCRT_Font_Release(textproperties.font);
			FSDK_SavePDFFile(autoWatermark.m_doc.GetDocument(), (outputFile + strFileName + "_Text.pdf").c_str());

		}
		//Image watermark
		else if (1 == icount)
		{
			FSDK_OutputLog("Begin to add image watermark.\r\n");
			CAutoImage autoImage((inputFilesPath + "/FoxitLogo.jpg").c_str(), "rb");
			if (!autoImage.GetFile())
				return FSCRT_ERRCODE_ERROR;

			if (autoImage.LoadImage() != FSCRT_ERRCODE_SUCCESS)
				return FSCRT_ERRCODE_ERROR;

			//Insert image to document.
			ret = autoWatermark.CreateFromImage(autoImage.GetImage());
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Watermark_CreateFromImage with error code %d.\r\n", ret); 
				FSDK_CHECK_LICENSE(ret);
				return ret;
			}

			ret = autoWatermark.InsertToPage();
			if (FSCRT_ERRCODE_SUCCESS != ret)
				return ret;
			//SaveAs document.
			FSDK_SavePDFFile(autoWatermark.m_doc.GetDocument(), (outputFile + strFileName + "_Image.pdf").c_str());
		}
		//Page watermark
		else if (2 == icount)
		{
			FSDK_OutputLog("Begin to add page watermark.\r\n");
			CAutoDocument markDoc((inputFilesPath + "/FoxitBigPreview.pdf").c_str(), "rb");
			if (!markDoc.GetFileHander())
				return FSCRT_ERRCODE_ERROR;

			//Load PDF document
			if (FSCRT_ERRCODE_SUCCESS != markDoc.LoadPDFDocument(NULL))
				return FSCRT_ERRCODE_ERROR;

			//Load page 0
			CAutoPDFPage markpage;
			ret = markpage.LoadPDFPage(markDoc.GetDocument(), 0);
			if (FSCRT_ERRCODE_SUCCESS != ret)
				return ret;

			ret = markpage.ParsePage();
			if (FSCRT_ERRCODE_SUCCESS != ret)
				return ret;

			//Insert page to document as watermark.
			ret = autoWatermark.CreateFromPage(markpage.GetPDFPage());
			if (FSCRT_ERRCODE_SUCCESS != ret)
			{
				FSDK_OutputLog("Failure: In function FSPDF_Watermark_CreateFromPage with error code %d.\r\n", ret); 
				FSDK_CHECK_LICENSE(ret);
				return ret;
			}

			ret = autoWatermark.InsertToPage();
			if (FSCRT_ERRCODE_SUCCESS != ret)
				return ret;

			FSDK_SavePDFFile(autoWatermark.m_doc.GetDocument(), (outputFile + strFileName + "_Page.pdf").c_str());
		}

		FSDK_OutputLog("Succeed to add watermark.\r\n\r\n");
	}
	return FSCRT_ERRCODE_SUCCESS;
}

void  InsertFolderWatermark(void* folder, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("success\r\n");
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		string outputFile = outputFilesPath + "/";
		if (strFile.size() > 0 && !isDir)
		{
			if (FSDK_CompareExtName(strFile.c_str(), ".pdf") && strcmp(strFile.c_str(), "FoxitTextAll_Insert.pdf"))
			{
				//Get file name without file ext.
				InsertFilewatermark(strFile, inputFilesPath, outputFile);
			}	
		}
		if (strFile.size() < 1) break;
	} while (1);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);

	//Get log File.
	string logPath = FSDK_GetLogFile("pdfwatermark");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfwatermark\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfwatermark\r\n\r\n");

	//Initialize library.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get output files folder path.
	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfwatermark");
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get input files folder.
	string inputFilesPath = FSDK_GetInputFilesFolder("pdfwatermark");
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());

	if (folder)
	{
		FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfwatermark: ");
		InsertFolderWatermark(folder, inputFilesPath, outputFilesPath);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		string filename = "AboutFoxit.pdf";
		string outputFile = outputFilesPath + "/";
		InsertFilewatermark(filename, inputFilesPath, outputFile);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
